==> openpgp-card-tools-debug-0.9.3-1-x86_64.pkg.tar.zst: usr/src/debug/openpgp-card-tools/openpgp-card-tools/src/commands/system.rs <==
// SPDX-FileCopyrightText: 2021-2023 Heiko Schaefer <heiko@schaefer.name>
// SPDX-FileCopyrightText: 2022 Nora Widdecke <mail@nora.pink>
// SPDX-License-Identifier: MIT OR Apache-2.0

use anyhow::{anyhow, Result};
use clap::{Parser, ValueEnum};
use openpgp_card_sequoia::{state::Open, Card};

use crate::util;

#[derive(Parser, Debug)]
pub struct SystemCommand {
    #[clap(subcommand)]
    pub cmd: SystemSubCommand,
}

#[derive(Parser, Debug)]
pub enum SystemSubCommand {
    /// Completely reset a card (deletes all data including keys!)
    FactoryReset {
        #[clap(
            name = "card ident",
            short = 'c',
            long = "card",
            help = "Identifier of the card to use"
        )]
        ident: String,
    },

    /// Change identity (Nitrokey Start only)
    ///
    /// A Nitrokey Start device contains three distinct virtual OpenPGP cards, select the identity
    /// of the virtual card to activate.
    SetIdentity {
        #[clap(
            name = "card ident",
            short = 'c',
            long = "card",
            help = "Identifier of the card to use"
        )]
        ident: String,

        /// Identity of the virtual card to activate
        #[clap(name = "identity", value_enum)]
        id: SetIdentityId,
    },

    /// Activate File (recover from "card terminated" state)
    ///
    /// If a card is in "Terminated" state, this command activates it again.
    /// Command can only be performed if only one card is connected to the
    /// system, and that card is in the terminated state.
    ///
    /// Otherwise an error message is printed.
    ActivateFile {},
}

pub fn system(command: SystemCommand) -> Result<(), Box<dyn std::error::Error>> {
    match command.cmd {
        SystemSubCommand::FactoryReset { ident } => factory_reset(&ident)?,
        SystemSubCommand::SetIdentity { ident, id } => set_identity(&ident, id)?,
        SystemSubCommand::ActivateFile {} => activate()?,
    }
    Ok(())
}

pub fn factory_reset(ident: &str) -> Result<()> {
    println!("Resetting Card {}", ident);
    let backend = util::open_card(ident)?;
    let mut open: Card<Open> = backend.into();

    let mut card = open.transaction()?;
    card.factory_reset().map_err(|e| anyhow!(e))
}

#[derive(ValueEnum, Debug, Clone)]
pub enum SetIdentityId {
    #[clap(name = "0")]
    Zero,
    #[clap(name = "1")]
    One,
    #[clap(name = "2")]
    Two,
}

impl From<SetIdentityId> for u8 {
    fn from(id: SetIdentityId) -> Self {
        match id {
            SetIdentityId::Zero => 0,
            SetIdentityId::One => 1,
            SetIdentityId::Two => 2,
        }
    }
}

pub fn set_identity(ident: &str, id: SetIdentityId) -> Result<(), Box<dyn std::error::Error>> {
    let backend = util::open_card(ident)?;
    let mut open: Card<Open> = backend.into();
    let mut card = open.transaction()?;

    card.set_identity(u8::from(id))?;
    Ok(())
}

pub fn activate() -> Result<(), Box<dyn std::error::Error>> {
    openpgp_card_pcsc::PcscBackend::activate_terminated_card()?;
    Ok(())
}

==> openpgp-card-tools-debug-0.10.0-2-x86_64.pkg.tar.zst: usr/src/debug/openpgp-card-tools/openpgp-card-tools/src/commands/system.rs <==
// SPDX-FileCopyrightText: 2021-2023 Heiko Schaefer <heiko@schaefer.name>
// SPDX-FileCopyrightText: 2022 Nora Widdecke <mail@nora.pink>
// SPDX-FileCopyrightText: 2023 David Runge <dave@sleepmap.de>
// SPDX-License-Identifier: MIT OR Apache-2.0

use anyhow::{anyhow, Result};
use card_backend_pcsc::PcscBackend;
use clap::{Parser, ValueEnum};
use openpgp_card_sequoia::types::Error;

use crate::util;

#[derive(Parser, Debug)]
pub struct SystemCommand {
    #[command(subcommand)]
    pub cmd: SystemSubCommand,
}

#[derive(Parser, Debug)]
pub enum SystemSubCommand {
    /// Completely reset a card (deletes all data including keys!)
    FactoryReset {
        #[arg(
            name = "card ident",
            short = 'c',
            long = "card",
            help = "Identifier of the card to use"
        )]
        ident: String,
    },

    /// Change identity (Nitrokey Start only)
    ///
    /// A Nitrokey Start device contains three distinct virtual OpenPGP cards, select the identity
    /// of the virtual card to activate.
    SetIdentity {
        #[arg(
            name = "card ident",
            short = 'c',
            long = "card",
            help = "Identifier of the card to use"
        )]
        ident: String,

        /// Identity of the virtual card to activate
        #[arg(name = "identity", value_enum)]
        id: SetIdentityId,
    },

    /// Activate File (recover from "card terminated" state)
    ///
    /// If a card is in "Terminated" state, this command activates it again.
    /// Command can only be performed if only one card is connected to the
    /// system, and that card is in the terminated state.
    ///
    /// Otherwise an error message is printed.
    ActivateFile {},
}

pub fn system(command: SystemCommand) -> Result<(), Box<dyn std::error::Error>> {
    match command.cmd {
        SystemSubCommand::FactoryReset { ident } => factory_reset(&ident)?,
        SystemSubCommand::SetIdentity { ident, id } => set_identity(&ident, id)?,
        SystemSubCommand::ActivateFile {} => activate()?,
    }
    Ok(())
}

pub fn factory_reset(ident: &str) -> Result<()> {
    eprintln!("Resetting Card {}", ident);
    let mut open = util::open_card(ident)?;
    let mut card = open.transaction()?;

    card.factory_reset().map_err(|e| anyhow!(e))
}

#[derive(ValueEnum, Debug, Clone)]
pub enum SetIdentityId {
    #[value(name = "0")]
    Zero,
    #[value(name = "1")]
    One,
    #[value(name = "2")]
    Two,
}

impl From<SetIdentityId> for u8 {
    fn from(id: SetIdentityId) -> Self {
        match id {
            SetIdentityId::Zero => 0,
            SetIdentityId::One => 1,
            SetIdentityId::Two => 2,
        }
    }
}

pub fn set_identity(ident: &str, id: SetIdentityId) -> Result<(), Box<dyn std::error::Error>> {
    let mut open = util::open_card(ident)?;
    let mut card = open.transaction()?;

    card.set_identity(u8::from(id))?;
    Ok(())
}

pub fn activate() -> Result<(), Box<dyn std::error::Error>> {
    let mut cards: Vec<_> = PcscBackend::card_backends(None)?.collect();

    if cards.len() != 1 {
        return Err(Error::InternalError(format!(
            "This command is only allowed if exactly one card is connected, found {}.",
            cards.len()
        ))
        .into());
    }

    let mut card = cards
        .pop()
        .ok_or_else(|| anyhow!("There are no more cards"))??;
    let mut tx = card.transaction(None)?;

    // SELECT OpenPGP
    let select_res = tx.transmit(
        &[
            0x00, 0xa4, 0x04, 0x00, 0x00, 0x00, 0x06, 0xd2, 0x76, 0x00, 0x01, 0x24, 0x01,
        ],
        254,
    )?;

    if select_res == [0x62, 0x85] {
        // ACTIVATE FILE
        let activate_res = tx.transmit(&[0x00, 0x44, 0x00, 0x00], 254)?;

        if activate_res == [0x90, 0x00] {
            Ok(())
        } else {
            Err(Error::InternalError(format!(
                "ACTIVATE FILE failed with status {:02x?}",
                activate_res
            ))
            .into())
        }
    } else {
        Err(Error::InternalError("Card doesn't appear to be terminated.".to_string()).into())
    }
}
